#!/usr/bin/env python
# -*- coding: utf-8 -*-
# taken from https://github.com/Elektordi/obs-websocket-py after running generate_classes.py to update the api

"""
Python library to communicate with an obs-websocket server.
"""

from .core import obsws  # noqa: F401
